@@ -9,7 +9,7 @@
  *  CODE-OCEC-09-009. All rights reserved.
  *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  For details, see <http://www.schedmd.com/slurmdocs/>.
  *  Please also read the included file: DISCLAIMER.
  *
  *  SLURM is free software; you can redistribute it and/or modify it under
@@ -39,7 +39,8 @@
 \*****************************************************************************/
 
 #include <stdio.h>
-#include <slurm/slurm_errno.h>
+
+#include "slurm/slurm_errno.h"
 
 #include "src/common/bitstring.h"
 #include "src/common/list.h"
@@ -114,6 +115,9 @@ extern List find_preemptable_jobs(struct job_record *job_ptr)
 		    (bit_overlap(job_p->node_bitmap,
 				 job_ptr->part_ptr->node_bitmap) == 0))
 			continue;
+		if (job_ptr->details &&
+		    (job_ptr->details->expanding_jobid == job_p->job_id))
+			continue;
 
 		/* This job is a preemption candidate */
 		if (preemptee_job_list == NULL) {
