@@ -8,7 +8,7 @@
  *  CODE-OCEC-09-009. All rights reserved.
  *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  For details, see <http://www.schedmd.com/slurmdocs/>.
  *  Please also read the included file: DISCLAIMER.
  *
  *  SLURM is free software; you can redistribute it and/or modify it under
@@ -42,7 +42,8 @@
 
 #include <unistd.h>
 #include <sys/types.h>
-#include <src/common/slurm_protocol_defs.h>
+#include "src/common/slurm_protocol_defs.h"
+#include "src/slurmctld/slurmctld.h"
 
 
 /* User RPC processing to set, get, clear, and pull triggers */
@@ -53,6 +54,8 @@ extern int trigger_pull(trigger_info_msg_t *msg);
 
 /* Note the some event has occured and flag triggers as needed */
 extern void trigger_block_error(void);
+extern void trigger_front_end_down(front_end_record_t *front_end_ptr);
+extern void trigger_front_end_up(front_end_record_t *front_end_ptr);
 extern void trigger_node_down(struct node_record *node_ptr);
 extern void trigger_node_drained(struct node_record *node_ptr);
 extern void trigger_node_failing(struct node_record *node_ptr);
